@@ -4,7 +4,7 @@ use Moose;
 use MooseX::StrictConstructor;
 use namespace::clean -except => 'meta';
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 extends 'Hailo::Storage::Mixin::DBD';
 
@@ -58,8 +58,8 @@ sub _add_expr {
 }
 
 sub _add_token {
-    my ($self, $token) = @_;
-    $self->sth->{add_token}->execute($token);
+    my ($self, $token_info) = @_;
+    $self->sth->{add_token}->execute(@$token_info);
     return $self->sth->{add_token}->fetchrow_array;
 }
 
@@ -76,29 +76,29 @@ L<DBD::Pg|DBD::Pg>
 
 As a module:
 
-    my $hailo = Hailo->new(
-        train_file    => 'hailo.trn',
-        storage_class => 'Pg',
-        storage_args  => {
-            dbname    => 'hailo',
-            host      => 'localhost',
-            port      => '5432',
-            options   => '...',
-            username  => 'hailo',
-            password  => 'hailo'
-        },
-    );
+ my $hailo = Hailo->new(
+     train_file    => 'hailo.trn',
+     storage_class => 'Pg',
+     storage_args => {
+         dbname   => 'hailo',
+         host     => 'localhost',
+         port     => '5432',
+         options  => '...',
+         username => 'hailo',
+         password => 'hailo'
+     },
+ );
 
 From the command line:
 
-    hailo --train        hailo.trn \
-          --storage      Pg \
-          --storage-args dbname=hailo \
-          --storage-args host=localhost \
-          --storage-args port=5432 \
-          --storage-args options=... \
-          --storage-args username=hailo \
-          --storage-args password=hailo
+ hailo --train hailo.trn \
+     --storage      Pg \
+     --storage-args dbname=hailo \
+     --storage-args host=localhost \
+     --storage-args port=5432 \
+     --storage-args options=... \
+     --storage-args username=hailo \
+     --storage-args password=hailo
 
 Almost all of these options can be omitted, see L<DBD::Pg's
 documentation|DBD::Pg/"connect"> for the default values.
@@ -109,6 +109,24 @@ See L<Hailo's documentation|Hailo> for other non-Pg specific options.
 
 This backend maintains information in a PostgreSQL database.
 
+=head1 ATTRIBUTES
+
+=head2 C<storage_args>
+
+This is a hash reference which can have the following keys:
+
+B<'dbname'>, the name of the database to use (required).
+
+B<'host'>, the host to connect to (required).
+
+B<'port'>, the port to connect to (required).
+
+B<'options'>, additional options to pass to PostgreSQL.
+
+B<'username'>, the username to use.
+
+B<'password'>, the password to use.
+
 =head1 CAVEATS
 
 It's around 8x-10x slower than L<the SQLite
@@ -130,5 +148,5 @@ it under the same terms as Perl itself.
 =cut
 
 __DATA__
-__[ query_exists_db ]__
+__[ static_query_exists_db ]__
 SELECT count(*) FROM information_schema.columns WHERE table_name ='info';
